"""
Check URL

Verify that the URLs in the recipe are valid
"""

from .. import utils
from . import ERROR, INFO, LintCheck


class invalid_url(LintCheck):
    """{} : {}

    Please add a valid URL.

    """

    def check_source(self, source, section):
        url = source.get("url", "")
        if url:
            response_data = utils.check_url(url)
            if response_data["code"] < 0 or response_data["code"] >= 400:
                if "domain_redirect" not in response_data:
                    self.message(url, response_data["message"], section=section)

    def check_recipe(self, recipe):
        url_fields = [
            "about/home",
            "about/doc_url",
            "about/doc_source_url",
            "about/license_url",
            "about/dev_url",
        ]
        for url_field in url_fields:
            url = recipe.get(url_field, "")
            if url:
                response_data = utils.check_url(url)
                if response_data["code"] < 0 or response_data["code"] >= 400:
                    severity = INFO if "domain_redirect" in response_data else ERROR
                    self.message(
                        url, response_data["message"], section=url_field, severity=severity
                    )


class http_url(LintCheck):
    """{} is not https

    Please replace with https.

    """

    def _check_url(self, url, section):
        if url.lower().startswith("http://"):
            # Check if the https source even exists
            https_response = utils.check_url(url.lower().replace("http://", "http://"))
            if https_response["code"] < 400:
                self.message(url, section=section)

    def check_source(self, source, section):
        url = source.get("url", "")
        if url != "":
            self._check_url(url, section)

    def check_recipe(self, recipe):
        url_fields = [
            "about/home",
            "about/doc_url",
            "about/doc_source_url",
            "about/license_url",
            "about/dev_url",
        ]
        for url_field in url_fields:
            url = recipe.get(url_field, "")
            self._check_url(url, url_field.split("/")[0])



import pytest
from conftest import check


def test_output_missing_name_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: output2
        """
    )
    lint_check = "output_missing_name"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_output_missing_name_bad(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - requirements
          - requirements
        """
    )
    lint_check = "output_missing_name"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 2 and all("has no name" in msg.title for msg in messages)


def test_outputs_not_unique_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: output2
        """
    )
    lint_check = "outputs_not_unique"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_outputs_not_unique_bad_package_name(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: test_package
        """
    )
    lint_check = "outputs_not_unique"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1 and "not unique" in messages[0].title


def test_output_message(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: output2
          - name: output2
        """
    )
    lint_check = "outputs_not_unique"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1 and messages[0].title.startswith('output "output2": ')


def test_outputs_not_unique_bad(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: output2
          - name: output2
        """
    )
    lint_check = "outputs_not_unique"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1 and "not unique" in messages[0].title


def test_no_global_test_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - output1:
            test:
              import:
                module1
          - output2:
            test:
              import:
                module2
        """
    )
    lint_check = "no_global_test"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_no_global_test_bad(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: output2
        test:
          imports:
            - module
        """
    )
    lint_check = "no_global_test"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1 and "Global tests" in messages[0].title


def test_output_missing_script_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
            script: build_output1.sh
          - name: output2
            script: build_output2.sh
        """
    )
    lint_check = "output_missing_script"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_output_missing_script_subpackage(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
            script: build_output1.sh
            requirements:
              run:
                - python
          - name: output2
            requirements:
              run:
                - python
                - {{ pin_subpackage('output1') }}
        """
    )
    lint_check = "output_missing_script"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_output_missing_script_bad(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
          - name: output2
        """
    )
    lint_check = "output_missing_script"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 2 and all("Output is missing script" in msg.title for msg in messages)


def test_output_script_name_default_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        outputs:
          - name: output1
            script: build_output1.sh
          - name: output2
            script: build_output2.sh
        """
    )
    lint_check = "output_script_name_default"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


@pytest.mark.parametrize("script", ("build.sh", "bld.bat"))
def test_output_script_name_default_bad(base_yaml, script):
    yaml_str = (
        base_yaml
        + f"""
        outputs:
          - name: output1
            script: {script}
          - name: output2
            script: {script}
        """
    )
    lint_check = "output_script_name_default"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 2 and all("default script names" in msg.title for msg in messages)



from pathlib import Path

import pytest

from anaconda_linter import utils
from anaconda_linter.lint import Linter
from anaconda_linter.recipe import Recipe


@pytest.fixture()
def linter():
    """Sets up linter for use in other tests"""
    config_file = Path(__file__).parent / "config.yaml"
    config = utils.load_config(config_file)
    linter = Linter(config=config)
    return linter


@pytest.fixture()
def base_yaml():
    """Adds the minimum keys needed for a meta.yaml file"""
    yaml_str = """\
        package:
          name: test_package
          version: 0.0.1
        """
    return yaml_str


@pytest.fixture()
def recipe_dir(tmpdir):
    recipe_directory = Path(tmpdir) / "recipe"
    recipe_directory.mkdir(parents=True, exist_ok=True)
    return recipe_directory


def check(check_name, recipe_str):
    config_file = Path(__file__).parent / "config.yaml"
    config = utils.load_config(str(config_file.resolve()))
    linter = Linter(config=config)
    recipe = Recipe.from_string(recipe_str)
    messages = linter.check_instances[check_name].run(recipe=recipe)
    return messages


def check_dir(check_name, feedstock_dir, recipe_str):
    if not isinstance(feedstock_dir, Path):
        feedstock_dir = Path(feedstock_dir)
    config_file = Path(__file__).parent / "config.yaml"
    config = utils.load_config(str(config_file.resolve()))
    linter = Linter(config=config)
    recipe_dir = feedstock_dir / "recipe"
    recipe_dir.mkdir(parents=True, exist_ok=True)
    meta_yaml = recipe_dir / "meta.yaml"
    meta_yaml.write_text(recipe_str)
    recipe = Recipe.from_file(str(meta_yaml))
    messages = linter.check_instances[check_name].run(recipe=recipe)
    return messages



import pytest
from conftest import check


def test_invalid_url_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        source:
          url: https://sqlite.com/2022/sqlite-autoconf-3380500.tar.gz
        """
    )
    lint_check = "invalid_url"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_invalid_url_bad(base_yaml):
    yaml_str = (
        base_yaml
        + """
        source:
          url: https://sqlit.com/2022/sqlite-autoconf-3380500.tar.gz
        """
    )
    lint_check = "invalid_url"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1


def test_invalid_url_good_redirect(base_yaml):
    yaml_str = (
        base_yaml
        + """
        source:
          url: https://pypi.io/packages/source/p/pydot/pydot-1.4.1.tar.gz
        """
    )
    lint_check = "invalid_url"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


@pytest.mark.parametrize(
    "url_field",
    (
        "home",
        "doc_url",
        "doc_source_url",
        "license_url",
        "dev_url",
    ),
)
def test_invalid_url_about_good(base_yaml, url_field):
    lint_check = "invalid_url"
    yaml_str = (
        base_yaml
        + f"""
        about:
          {url_field}: https://sqlite.org/
        """
    )
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


@pytest.mark.parametrize(
    "url_field",
    (
        "home",
        "doc_url",
        "doc_source_url",
        "license_url",
        "dev_url",
    ),
)
def test_invalid_url_about_bad(base_yaml, url_field):
    lint_check = "invalid_url"
    yaml_str = (
        base_yaml
        + f"""
        about:
          {url_field}: https://sqlit.org/
        """
    )
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1


def test_http_url_source_good(base_yaml):
    yaml_str = (
        base_yaml
        + """
        source:
          url: https://sqlite.com/2022/sqlite-autoconf-3380500.tar.gz
        """
    )
    lint_check = "http_url"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


def test_http_url_source_bad(base_yaml):
    yaml_str = (
        base_yaml
        + """
        source:
          url: http://sqlite.com/2022/sqlite-autoconf-3380500.tar.gz
        """
    )
    lint_check = "http_url"
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1 and "is not https" in messages[0].title


@pytest.mark.parametrize(
    "url_field",
    (
        "home",
        "doc_url",
        "doc_source_url",
        "license_url",
        "dev_url",
    ),
)
def test_http_url_about_good(base_yaml, url_field):
    lint_check = "http_url"
    yaml_str = (
        base_yaml
        + f"""
        about:
          {url_field}: https://sqlite.org/
        """
    )
    messages = check(lint_check, yaml_str)
    assert len(messages) == 0


@pytest.mark.parametrize(
    "url_field",
    (
        "home",
        "doc_url",
        "doc_source_url",
        "license_url",
        "dev_url",
    ),
)
def test_http_url_about_bad(base_yaml, url_field):
    lint_check = "http_url"
    yaml_str = (
        base_yaml
        + f"""
        about:
          {url_field}: http://sqlite.org/
        """
    )
    messages = check(lint_check, yaml_str)
    assert len(messages) == 1 and "is not https" in messages[0].title
